import {ipcMain, shell, BrowserWindow} from 'electron';
import EventEmitter from 'events';

import urls from 'common/utils/urls';
import prefs from 'browser/utils/prefs';

class IpcListenersManager extends EventEmitter {

  constructor (mainWindowManager) {
    super();
    this.mainWindowManager = mainWindowManager;
  }

  /**
   * Bind events to local methods.
   */
  set () {
    ipcMain.on('notif-count', ::this.onNotifCount);
    ipcMain.on('close-window', ::this.onCloseWindow);
    ipcMain.on('open-url', ::this.onOpenUrl);
  }

  /**
   * Called when the 'notif-count' event is received.
   */
  onNotifCount (event, count, badgeDataUrl) {
    log('on renderer notif-count', count, !!badgeDataUrl || null);
    clearTimeout(this._delayedRemoveBadge);
    if (count) {
      this.mainWindowManager.notifCountChanged(count, badgeDataUrl);
    } else {
      this._delayedRemoveBadge = setTimeout(() => {
        this.mainWindowManager.notifCountChanged(count, badgeDataUrl);
      }, 1500);
    }
  }

  /**
   * Called when the 'close-window' event is received.
   */
  onCloseWindow () {
    this.mainWindowManager.window.close();
  }

  /**
   * Called when the 'open-url' event is received.
   */
  onOpenUrl (event, url, options) {
    url = urls.skipFacebookRedirect(url);

    if (urls.isDownloadUrl(url)) {
      log('on renderer open-url, downloading', url);
      this.mainWindowManager.window.loadURL(url);
    } else if (prefs.get('links-in-browser')) {
      log('on renderer open-url, externally', url);
      shell.openExternal(url);
    } else {
      log('on renderer open-url, new window', url);
      const newWindow = new BrowserWindow(options);
      newWindow.loadURL(url);
    }
  }

}

export default IpcListenersManager;
